﻿//--------------------------------------------------------------------------------------
// Copyright (c) Microsoft Corp. 
//
// File: Bubble.cs
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use this 
// file except in compliance with the License. You may obtain a copy of the License at 
// http://www.apache.org/licenses/LICENSE-2.0  
//  
// THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
// EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR 
// CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//  
// See the Apache Version 2.0 License for specific language governing permissions and 
// limitations under the License.
//--------------------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;
using Windows.UI.Xaml.Shapes;

namespace Bubbles
{
    public class Bubble
    {
        const int MinBubbleSize = 70;
        const int MaxBubbleSize = 100;
        

        public Storyboard AnimationStoryboard { get; set; }
        public Ellipse BubbleEllipse { get; set; }
        public bool Popped { get; set; }

        public DoubleAnimation XAnimation { get; set; }
        public DoubleAnimation YAnimation { get; set; }
        public double DestinationX { get; set; }
        public double DestinationY { get; set; }

        public Bubble(Point point)
        {
            createBubble(point);
        }

        private void createBubble(Point origin)
        {
            Point destination;
            Ellipse bubble = new Ellipse();
            int size = NextSize;
            origin.Y -= size / 2;

            DestinationX = origin.X;
            DestinationY = BubbleResources.NextYPoint(origin.Y);
            destination = new Point(DestinationX, DestinationY);

            bubble.Height = size;
            bubble.Width = size;
            bubble.Fill = NextFillColor;
            bubble.Opacity = NextOpacity;
            bubble.RenderTransform = BubbleResources.CreateTransformGroup(origin);
            bubble.RenderTransformOrigin = origin;
            bubble.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Top;
            bubble.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Left;
            bubble.PointerPressed += new Windows.UI.Xaml.Input.PointerEventHandler(bubble_MouseLeftButtonDown);
            BubbleEllipse = bubble;

            Popped = false;

            CreateBubbleStoryboard(bubble, origin, destination);
        }

        internal void CreateBubbleStoryboard(UIElement ellipse, Point origin, Point destination)
        {
            Storyboard sb = new Storyboard();
            sb.AutoReverse = false;

            Duration duration = StoryBoardDuration;

            XAnimation = BubbleResources.createAnimation(origin.X, destination.X, duration, ellipse, "(UIElement.RenderTransform).(TransformGroup.Children)[0].(TranslateTransform.X)");
            sb.Children.Add(XAnimation);

            YAnimation = BubbleResources.createAnimation(origin.Y, destination.Y, duration, ellipse, "(UIElement.RenderTransform).(TransformGroup.Children)[0].(TranslateTransform.Y)");
            sb.Children.Add(YAnimation);

            AnimationStoryboard = sb;
        }

        void bubble_MouseLeftButtonDown(object sender, Windows.UI.Xaml.Input.PointerRoutedEventArgs e)
        {
            Ellipse bubble = (Ellipse)sender;
            bubble.Visibility = Windows.UI.Xaml.Visibility.Collapsed;
            Popped = true;
        }

        internal static Duration StoryBoardDuration
        {
            get
            {
                return new Duration(TimeSpan.FromMilliseconds(1000 / BubbleResources.NumberFramesPerStoryboard));
            }
        }

        static SolidColorBrush NextFillColor
        {
            get
            {
                switch (BubbleResources.GetNextRandom(0, 5))
                {
                    case 0:
                        return new SolidColorBrush(Colors.Blue);
                    case 1:
                        return new SolidColorBrush(Colors.Magenta);
                    case 2:
                        return new SolidColorBrush(Colors.Green);
                    case 3:
                        return new SolidColorBrush(Colors.Red);
                    default:
                        return new SolidColorBrush(Colors.Yellow);
                }
            }
        }

        static double NextOpacity
        {
            get
            {
                double opacity = BubbleResources.GetNextRandom(50, 100);
                return opacity / 100;
            }
        }

        static int NextSize
        {
            get
            {
                return BubbleResources.GetNextRandom(MinBubbleSize, MaxBubbleSize);
            }
        }

    }
}
