﻿//--------------------------------------------------------------------------------------
// Copyright (c) Microsoft Corp. 
//
// File: BubbleResources.cs
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use this 
// file except in compliance with the License. You may obtain a copy of the License at 
// http://www.apache.org/licenses/LICENSE-2.0  
//  
// THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
// EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR 
// CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//  
// See the Apache Version 2.0 License for specific language governing permissions and 
// limitations under the License.
//--------------------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;
using Windows.UI.Xaml.Shapes;

namespace Bubbles
{
    public static class BubbleResources
    {
        const int XVelocityScaler = 7;
        const int YVelocityScaler = 6;

        static Random random = new Random();

        public static double NumberFramesPerStoryboard { get { return 15; } }

        public static int GetNextRandom(int min, int max)
        {
            return random.Next(min, max);
        }

        internal static DoubleAnimation createAnimation(double from, double to, Duration duration, UIElement target, string targetProperty)
        {
            DoubleAnimation animation = new DoubleAnimation();
            animation.Duration = duration;
            animation.From = from;
            animation.To = to;
            Storyboard.SetTarget(animation, target);
            Storyboard.SetTargetProperty(animation, targetProperty);
            return animation;
        }

        internal static double NextXPoint(double? oldX)
        {
            double FrameMultiplier = Math.Round(1000 / NumberFramesPerStoryboard, 1);
            double multiplier = Math.Round(FrameMultiplier / 16.7, 1);
            return GetNextRandom((int)(oldX - (XVelocityScaler * multiplier)), (int)(oldX + (XVelocityScaler * multiplier)));
        }

        internal static double NextYPoint(double? oldY)
        {
            ////creating a new random with a different seed value
            //Random rand = new Random((int)oldY);
            double FrameMultiplier = Math.Round(1000 / NumberFramesPerStoryboard, 1);
            double multiplier = Math.Round(FrameMultiplier / 16.7, 1);
            return GetNextRandom((int)(oldY - (YVelocityScaler * multiplier)), (int)(oldY - ((YVelocityScaler / 2) * multiplier)));
        }



        internal static TransformGroup CreateTransformGroup(Point point)
        {
            TransformGroup group = new TransformGroup();
            TranslateTransform translate = new TranslateTransform();
            translate.X = point.X;
            translate.Y = point.Y;
            group.Children.Add(translate);

            return group;
        }

    }
}
