﻿//--------------------------------------------------------------------------------------
// Copyright (c) Microsoft Corp. 
//
// File: MainPage.xaml.cs
//
// Licensed under the Apache License, Version 2.0 (the "License"); you may not use this 
// file except in compliance with the License. You may obtain a copy of the License at 
// http://www.apache.org/licenses/LICENSE-2.0  
//  
// THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
// EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR 
// CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT. 
//  
// See the Apache Version 2.0 License for specific language governing permissions and 
// limitations under the License.
//--------------------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Bubbles;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Animation;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Xaml.Shapes;
using System.Diagnostics.Tracing;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace BubblePopper
{

    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        const int NumberBubbleTubles = 4;
        const int padding = 100;

        const int MinBubbleGap = 2500;
        const int MaxBubbleGap = 5000;

        Dictionary<DispatcherTimer, BubbleTube> bubbleTubes;
        Dictionary<Storyboard, Bubble> bubbles;

        TimeSpan nextBubbleInterval
        {
            get
            {
                return TimeSpan.FromMilliseconds(BubbleResources.GetNextRandom(MinBubbleGap, MaxBubbleGap));
            }
        }

        public MainPage()
        {
            this.InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);

            bubbleTubes = new Dictionary<DispatcherTimer, BubbleTube>();
            bubbles = new Dictionary<Storyboard, Bubble>();
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            Duration duration = new Duration(new TimeSpan(0, 0, 0, 50));

            Storyboard waitForUIInitialization = new Storyboard();
            waitForUIInitialization.AutoReverse = false;
            waitForUIInitialization.Completed += initializeUI;
            waitForUIInitialization.Begin();
        }

        void initializeUI(object sender, object e)
        {
            double spacing = (LayoutRoot.ActualWidth) / NumberBubbleTubles;
            double startX = (LayoutRoot.ActualWidth / NumberBubbleTubles) - padding * 2;
            

            for (int x = 0; x < NumberBubbleTubles; x++)
            {
                DispatcherTimer nextBubbleTimer = new DispatcherTimer();
                double x1 = startX + (spacing * x);
                double bottomY = this.ActualHeight - rectGround.ActualHeight - BubbleTube.TubeHeight;
                Point point = new Point(x1, bottomY);
                TransformGroup transform = BubbleResources.CreateTransformGroup(point);
                BubbleTube bubbler = new BubbleTube(transform, point);

                bubbler.VerticalAlignment = Windows.UI.Xaml.VerticalAlignment.Top;
                bubbler.HorizontalAlignment = Windows.UI.Xaml.HorizontalAlignment.Left;

                

                LayoutRoot.Children.Add(bubbler);

                nextBubbleTimer = new DispatcherTimer();
                nextBubbleTimer.Interval = TimeSpan.FromMilliseconds(BubbleResources.GetNextRandom(500, MinBubbleGap));
                nextBubbleTimer.Tick += makeBubbles;
                nextBubbleTimer.Start();
                bubbleTubes.Add(nextBubbleTimer, bubbler);
            }

        }

        void makeBubbles(object sender, object e)
        {
            DispatcherTimer timer = (DispatcherTimer)sender;
            BubbleTube bubbler = (BubbleTube)bubbleTubes[timer];
            Point p = bubbler.Point;

            p.Y -= bubbler.ActualHeight;
            createBubble(p);

            timer.Interval = nextBubbleInterval;
        }


        private void createBubble(Point origin)
        {
            Bubble bubble = new Bubble(origin);

            bubble.AnimationStoryboard.Completed += moveBubble_Completed;
            LayoutRoot.Children.Add(bubble.BubbleEllipse);
            bubbles.Add(bubble.AnimationStoryboard, bubble);
            bubble.AnimationStoryboard.Begin();
        }



        void moveBubble_Completed(object sender, object e)
        {
            Storyboard oldStoryBoard = (Storyboard)sender;
            Bubble bubble = (Bubble)bubbles[oldStoryBoard];
            
            //TimelineCollection tlc = oldStoryBoard.Children;
            //DoubleAnimation xAnimation = (DoubleAnimation)tlc[0];
            //DoubleAnimation yAnimation = (DoubleAnimation)tlc[1];

            double? xTo = bubble.DestinationX; //xAnimation.To;
            double? yTo = bubble.DestinationY; //yAnimation.To;
            double nextX = BubbleResources.NextXPoint(xTo);
            double nextY = BubbleResources.NextYPoint(yTo);
            //Point origin;
            //Point destination;

            //Check to see if the buble is off the top of of the screen
            if (yTo < 0 - bubble.BubbleEllipse.ActualHeight)
            {
                //Remove the old storyboard
                bubbles.Remove(bubble.AnimationStoryboard);

                //Remove the ellipse from the XAML tree
                LayoutRoot.Children.Remove(bubble.BubbleEllipse);
                return;
            }

            //origin = new Point((double)xTo, (double)yTo);
            //destination = new Point(nextX, nextY);

            ////create the new storyboard
            //bubble.CreateBubbleStoryboard(bubble.BubbleEllipse, origin, destination);

            //bubbles.Add(bubble.AnimationStoryboard, bubble);
            //bubble.AnimationStoryboard.Completed += moveBubble_Completed;

            bubble.XAnimation.From = bubble.DestinationX;
            bubble.XAnimation.To = nextX;
            bubble.YAnimation.From = bubble.DestinationY;
            bubble.YAnimation.To = nextY;

            bubble.DestinationX = nextX;
            bubble.DestinationY = nextY;

            bubble.AnimationStoryboard.Begin();
        }
    }
}
